#!/usr/bin/perl

use strict;
use warnings;

sub sumup {
    (   my $tfh,
        my $queryNoCheck,
        my $totalAccuracy,
        my $totalTime,
        my $totalCount,
        my $gt
    ) = @_;
    my $resultNo      = 0;
    my $relevantCount = 0;
    my $dataCount     = 0;
    my $queryNo;
    my $epsilon;
    my $queryTime;

    while (<$tfh>) {
        chomp($_);
        if (/^#/) {
            my @tf = split( "=", $_ );
            if (/^# Query No\./) {
                $queryNo = $tf[1];
            }
            elsif (/^# Epsilon/) {
                $epsilon = $tf[1];
            }
            elsif (/^# Query Time \(msec\)/) {
                $queryTime = $tf[1];
            }
            elsif (/^# End of Search/) {
                $resultNo++;
                my $accuracy
                    = $dataCount > 0 ? $relevantCount / $dataCount : 0;

#print "# $$queryNo\t$resultNo\t$epsilon\t$dataCount\t$relevantCount\t$accuracy\t$queryTime\n";
                $$totalAccuracy{$epsilon} += $accuracy;
                $$totalTime{$epsilon}     += $queryTime;
                $$totalCount{$epsilon}++;
                $relevantCount = 0;
                $dataCount     = 0;
            }
            elsif (/^# End of Query/) {
                last;
            }
        }
        else {
            my @result = split( / |\t/, $_ );
            my $rank   = $result[0];
            my $id     = $result[1];
            if ( exists $$gt{$id} ) {
                $relevantCount++;
            }
            $dataCount++;
            if ( $rank != $dataCount ) {
                die
                    "inner error! queryNo:$queryNoCheck rank:$rank dataCount:$dataCount !!\n";
            }
        }
    }
}

sub checkAndGetSize {
    ( my $target ) = @_;

    open( TFILE, "< $target" ) or die("error :$!");

    my $lineCount = 0;
    my $dataCount = 0;
    my $dataSize  = 0;
    my $lastDataLine;
    while (<TFILE>) {
        $lineCount++;
        chomp($_);
        if (/^# Query No\./) {
            $dataCount = 0;
        }
        elsif (/^# End of Search/) {
            if ( $dataSize == 0 ) {
                $dataSize = $dataCount;
            }
            elsif ( $dataSize != $dataCount ) {
                die "Result sizes are inconsistent! $dataSize:$dataCount\n",
                    "  Line No.$lineCount : $lastDataLine\n";
            }
        }
        elsif (/^[^#]/) {
            my @result = split( / |\t/, $_ );
            my $rank = $result[0];
            $dataCount++;
            if ( $rank != $dataCount ) {
                die "check: inner error! $rank $dataCount !!\n";
            }
        }
        $lastDataLine = $_;
    }
    close(TFILE);
    return $dataSize;
}

sub loadGroundTruth {
    ( my $gtfh, my $resultDataSize, my $gt ) = @_;
    my $dataCount   = 0;
    my $searchCount = 0;
    while (<$gtfh>) {
        chomp($_);
        if (/^# End of Search/) {
            $searchCount++;
        }
        elsif (/^# End of Query/) {
            if ( $searchCount != 1 ) {
                die "Error: gt has not just one search result.\n";
            }
            elsif ( $dataCount < $resultDataSize ) {
                die
                    "Error: gt data is less than result size! $dataCount:$resultDataSize\n";
            }
            last;
        }
        elsif ( /^[^#]/ && ++$dataCount <= $resultDataSize ) {
            my @result = split( / |\t/, $_ );
            my $id = $result[1];
            $$gt{$id} = 1;
        }
    }
}

sub main {
    if ( @ARGV != 2 ) {
        die "evalngt.pl ground-truth search-result\n";
    }

    my $gt     = $ARGV[0];
    my $target = $ARGV[1];

    my $resultDataSize = checkAndGetSize($target);

    open( my $gtfh, "< $gt" )     or die("error :$!");
    open( my $tfh,  "< $target" ) or die("error :$!");

    my $queryNo       = 1;
    my %totalAccuracy = ();
    my %totalCount    = ();
    my %totalTime     = ();

    while (<$gtfh>) {
        chomp($_);
        if (/^# Query No\./) {
            my @f = split( "=", $_ );
            if ( $f[1] == $queryNo ) {
                my %gt = ();
                loadGroundTruth( $gtfh, $resultDataSize, \%gt );
                sumup( $tfh, $queryNo, \%totalAccuracy, \%totalTime,
                    \%totalCount, \%gt );
                $queryNo++;
            }
        }
    }
    close($gtfh);
    close($tfh);

    print "Ground Truth=$gt\n";
    print "Target=$target\n";
    print "Epsilon\t# of Queries\tAccuracy\tTime(msec)\n";
    foreach my $k ( sort { $a <=> $b } keys(%totalAccuracy) ) {
        my $averageAccuracy = $totalAccuracy{$k} / $totalCount{$k};
        my $averageTime     = $totalTime{$k} / $totalCount{$k};
        print "$k\t$totalCount{$k}\t$averageAccuracy\t$averageTime\n";
    }
}

if (caller) {
    1;
}
else {
    &main;
}
