#!/bin/bash -x

readonly ROOT_DIR=$(dirname $0)

readonly DATA_DIR="$ROOT_DIR/data"
readonly TRAIN_DATA="$DATA_DIR/train.txt"
readonly TEST_DATA="$DATA_DIR/test.txt"
readonly BIN_DIR="$ROOT_DIR/bin"
readonly NGT_BIN="/usr/local/bin/ngt"
readonly ENV_BIN="$BIN_DIR/env.sh"
readonly FLANN_CREATE_BIN="$BIN_DIR/flann_create"
readonly FLANN_SEARCH_BIN="$BIN_DIR/flann_search"
readonly SASH_CREATE_BIN="$BIN_DIR/sash_create"
readonly SASH_SEARCH_BIN="$BIN_DIR/sash_search"
readonly EVAL_BIN="$BIN_DIR/eval.pl"

readonly OUTPUT_DIR="$ROOT_DIR/output"
readonly EVAL_DATA="$OUTPUT_DIR/ground_truth.txt"
readonly NGT_INDEX="$OUTPUT_DIR/ngt_index"
readonly FLANN_INDEX="$OUTPUT_DIR/flann_index"
#readonly SASH_INDEX="$DATA_DIR/train.txt.sash"
readonly NGT_RESULT="$OUTPUT_DIR/ngt_result.txt"
readonly FLANN_RESULT="$OUTPUT_DIR/flann_result.txt"
readonly SASH_RESULT="$OUTPUT_DIR/sash_result.txt"
readonly NGT_EVAL="$OUTPUT_DIR/ngt_eval.txt"
readonly FLANN_EVAL="$OUTPUT_DIR/flann_eval.txt"
readonly SASH_EVAL="$OUTPUT_DIR/sash_eval.txt"

ERROR_FLAG=0

function check {
  if [ ! -e $1 ]; then
    echo "$1 is not found." >&2
    ERROR_FLAG=1
  fi
}

function main {

  check $TRAIN_DATA
  check $TEST_DATA
  check $NGT_BIN
  check $FLANN_CREATE_BIN
  check $FLANN_SEARCH_BIN
  check $SASH_CREATE_BIN
  check $SASH_SEARCH_BIN
  check $EVAL_BIN

  if [ "$ERROR_FLAG" != "0" ]; then
    echo "Should run 'make' command before this command." >&2
    exit 1
  fi

  if [ ! -e $OUTPUT_DIR ]; then
    mkdir $OUTPUT_DIR
  fi

  source $ENV_BIN

  set -x

  $NGT_BIN create -d 50 $NGT_INDEX $TRAIN_DATA

  # create ground truth data
  $NGT_BIN search -i s -n 10 -o e $NGT_INDEX $TEST_DATA > $EVAL_DATA

  $NGT_BIN search -n 10 -e 0.0:0.2:0.025 -o e $NGT_INDEX $TEST_DATA > $NGT_RESULT

  $FLANN_CREATE_BIN $TRAIN_DATA $FLANN_INDEX

  $FLANN_SEARCH_BIN $TRAIN_DATA $FLANN_INDEX $TEST_DATA > $FLANN_RESULT

  $SASH_CREATE_BIN $TRAIN_DATA

  $SASH_SEARCH_BIN $TRAIN_DATA $TEST_DATA > $SASH_RESULT
 
  $EVAL_BIN $EVAL_DATA $NGT_RESULT > $NGT_EVAL

  $EVAL_BIN $EVAL_DATA $FLANN_RESULT > $FLANN_EVAL

  $EVAL_BIN $EVAL_DATA $SASH_RESULT > $SASH_EVAL

  set +x

  echo -e "\nNGT evaluation below:"
  cat $NGT_EVAL

  echo -e "\nFLANN evaluation below:"
  cat $FLANN_EVAL

  echo -e "\nSASH evaluation below:"
  cat $SASH_EVAL

  return ${?}
}

main ${@}
exit ${?}

