
#include <flann/flann.hpp>
#include <string>
#include <fstream>

#include <stdio.h>

#include "utility.hpp"

typedef float DataType;

using namespace flann;
using namespace std;
using namespace utility;

int main(int argc, char** argv) {
  string ssvfile = "";
  string indexfile = "";
  string testfile = "";
  int start = 1000;
  int end = 6000;
  float step = 1.2;
  int nn = 10;
  bool help = false;

  int result;
  while ((result = getopt(argc, argv, "f:e:s:n:h")) != -1) {
    switch (result) {
      case 'f':
        start = atoi(optarg);
        break;
      case 'e':
        end = atoi(optarg);
        break;
      case 's':
        step = atof(optarg);
        break;
      case 'n':
        nn = atoi(optarg);
        break;
      case 'h':
        help = true;
        break;
    }
  }

  if (argc - optind < 2 || help) {
    cerr << "Usage: ./flann_search [options] train.ssv index test.ssv" << endl;
    cerr << " -f INT       Set a first of accuracy parameter" << endl;
    cerr << " -e INT       Set a end of accuracy parameter" << endl;
    cerr << " -s FLOAT     Set a step of accuracy parameter" << endl;
    cerr << " -n INT       Set the number of vectors for accuary" << endl;
    cerr << " -h           Show this message" << endl;
    cerr << endl;
    cerr << "Example: ./flann_search -f 1000 -e 6000 -s 1.2 -n 10 train.ssv "
            "test.ssv" << endl;
    exit(-1);
  }
  ssvfile = argv[optind++];
  indexfile = argv[optind++];
  testfile = argv[optind++];

  cerr << "data file=" << ssvfile << endl;
  cerr << "index file=" << indexfile << endl;
  cerr << "query file=" << testfile << endl;
  cerr << "k of knn=" << nn << endl;

  size_t dim = 0;
  size_t size = get_data_size(ssvfile, dim);

  cerr << "load data, size=" << size << " dim=" << dim << endl;

  vector<DataType> data;
  data.reserve(size * dim);
  load_ssv(ssvfile, data);

  // matrix(data, rows, columns);
  Matrix<DataType> dataset(&data[0], size, dim);

  size_t qsize = get_data_size(testfile, dim);
  vector<DataType> qdata;
  qdata.reserve(qsize * dim);
  load_ssv(testfile, qdata);

  Timer timer;

  Matrix<int> indices(new int[dim * nn], dim, nn);
  Matrix<DataType> dists(new DataType[dim * nn], dim, nn);

  Index<L2<DataType> > index(dataset, SavedIndexParams(indexfile));
  for (int n = 0; n < size; n++) {
    Matrix<DataType> q(&qdata[n * dim], 1, dim);
    for (int f = start; f < end; f *= step) {
      cout << "# Query No.=" << n + 1 << endl;
      cout << "# Size=" << size << endl;
      cout << "# Epsilon=" << f << endl;
      timer.reset();
      timer.start();

      index.knnSearch(q, indices, dists, nn, flann::SearchParams(f, 0, true));

      timer.stop();
      cout << "# Query Time (msec)=" << timer.time*(double)1000.0 << endl;
      for (int i = 0; i < nn; i++) {
        cout << i + 1 << "\t" << indices[0][i] + 1 << "\t(null)\t"
             << dists[0][i] << endl;
      }
      cout << "# End of Search" << endl;
    }
    cout << "# End of Query" << endl;
  }

  return 0;
}
