#include <string>

#include "Sash.h"
#include "DenseVecData.h"

#include "utility.hpp"

using namespace std;
using namespace utility;

int main(int argc, char* argv[]) {
  string ssvfile = "";
  int numParents = 6;
  bool help = false;

  int result;
  while ((result = getopt(argc, argv, "c:h")) != -1) {
    switch (result) {
      case 'c':
        numParents = atoi(optarg);
        break;
      case 'h':
        help = true;
        break;
    }
  }

  if (argc - optind < 1 || help) {
    cerr << "Usage: ./runsash [options] data.ssv" << endl;
    cerr << " -c INT       Set an num parents parameter" << endl;
    cerr << " -h           Show this message" << endl;
    cerr << endl;
    cerr << "Example: ./sash_create -c 6 train.ssv" << endl;
    exit(-1);
  }
  ssvfile = argv[optind];

  Timer timer;

  Sash* dataSash;
  DistData** dataset;
  size_t dim = 0;
  vector<float> data;

  size_t size = get_data_size(ssvfile, dim);

  cerr << "load ssv" << endl;
  data.reserve(size * dim);
  load_ssv(ssvfile, data);
  dataset = new DistData* [size];
  for (size_t i = 0; i < size; i++) {
    dataset[i] = new DenseVecData(&data[i * dim], dim);
  }

  // hash => 12345UL
  dataSash = new Sash(12345UL);
  // debug option disable
  dataSash->setVerbosity(0);

  cerr << "build start: size:" << size << ", dimension:" << dim << endl;
  timer.start();

  dataSash->build(dataset, (int)size, numParents);
  dataSash->saveToFile(
      const_cast<char*>(ssvfile.c_str()));  // save to ssvfile + .sash

  timer.stop();
  cerr << "build end:" << timer << endl;

  return 0;
}
