#include <iostream>
#include <string>
#include <cstdlib>
#include <sstream>
#include <vector>
#include <fstream>
#include <sys/time.h>
#include <iomanip>

#include "Sash.h"
#include "DenseVecData.h"
#include "utility.hpp"

using namespace std;
using namespace utility;

int main(int argc, char* argv[]) {
  string ssvfile = "";
  string testfile = "";
  float start = 0.2;
  float end = 4.0;
  float step = 1.4;
  int topN = 10;
  bool help = false;

  int result;
  while ((result = getopt(argc, argv, "f:e:s:n:h")) != -1) {
    switch (result) {
      case 'f':
        start = atof(optarg);
        break;
      case 'e':
        end = atof(optarg);
        break;
      case 's':
        step = atof(optarg);
        break;
      case 'n':
        topN = atoi(optarg);
        break;
      case 'h':
        help = true;
        break;
    }
  }

  if (argc - optind < 2 || help) {
    cerr << "Usage: ./sash_search [options] train.ssv test.ssv" << endl;
    cerr << " -f FLOAT     Set a first of accuracy parameter" << endl;
    cerr << " -e FLOAT     Set a end of accuracy parameter" << endl;
    cerr << " -s FLOAT     Set a step of accuracy parameter" << endl;
    cerr << " -n INT       Set the number of vectors for accuary" << endl;
    cerr << " -h           Show this message" << endl;
    cerr << endl;
    cerr << "Example: ./sash_search -f 10 train.ssv test.ssv" << endl;
    exit(-1);
  }
  ssvfile = argv[optind++];
  testfile = argv[optind++];

  Timer timer;

  Sash* dataSash;
  vector<float> data;
  DistData** dataset;
  size_t dim = 0;
  size_t size = get_data_size(ssvfile, dim);
  data.reserve(size * dim);
  load_ssv(ssvfile, data);
  dataset = new DistData* [size];
  for (size_t i = 0; i < size; i++) {
    dataset[i] = new DenseVecData(&data[i * dim], dim);
  }
  cerr << "load sash index" << endl;
  dataSash = new Sash();
  dataSash->setVerbosity(0);
  dataSash->build(const_cast<char*>(ssvfile.c_str()), dataset, (int)size);

  cerr << "load test data" << endl;
  size_t tsize = get_data_size(testfile, dim);
  vector<float> tdata;
  DistData** tdataset;

  tdata.reserve(tsize * dim);
  load_ssv(testfile, tdata);
  tdataset = new DistData* [tsize];
  for (size_t i = 0; i < tsize; i++) {
    tdataset[i] = new DenseVecData(&tdata[i * dim], dim);
  }

  // Calculate accuracy
  float* approxDistList = new float[topN];
  int* approxIndexList = new int[topN];

  cerr << "start evaluation" << endl;

  for (int i = 0; i < tsize; i++) {
    for (float j = start; j < end; j *= step) {
      cout << "# Query No.=" << i + 1 << endl;
      cout << "# Size=" << tsize << endl;
      cout << "# Epsilon=" << j << endl;
      int resultSize;

      timer.reset();
      timer.start();

      dataSash->findNear(tdataset[i], topN, j);

      timer.stop();

      dataSash->getResultDists(approxDistList, topN);
      dataSash->getResultIndices(approxIndexList, topN);
      resultSize = dataSash->getResultNumFound();

      cout << "# Query Time (msec)=" << timer.time*(double)1000.0 << endl;
      for (int k = 0; k < resultSize; k++) {
        // queryNum id dist words
        cout << k + 1 << "\t" << approxIndexList[k] + 1 << "\t"
             << approxDistList[k] << "\t(null)\t" << endl;
      }
      cout << "# End of Search" << endl;
    }
    cout << "# End of Query=" << endl;
  }

  return 0;
}
