#include "utility.hpp"

namespace utility {
void split(string &str, char delim, vector<string> &vec) {
  stringstream ss(str);
  string s;
  while (getline(ss, s, delim)) {
    vec.push_back(s);
  }
}

int load_ssv(const string &ssvfile, vector<float> &data) {
  ifstream in(ssvfile.c_str());
  if (!in.is_open()) {
    cerr << "cannot open " << ssvfile << endl;
    exit(-1);
  }
  string line;
  vector<string> strs;
  while (getline(in, line)) {
    if (strlen(line.c_str()) == 0) {
      continue;
    }
    strs.clear();
    split(line, ' ', strs);
    for (vector<string>::iterator s = strs.begin(); s != strs.end(); s++) {
      data.push_back(atof(s->c_str()));
    }
  }
  in.close();

  return 0;
}

size_t get_data_size(const string &ssvfile, size_t &dim) {
  ifstream in(ssvfile.c_str());
  if (!in.is_open()) {
    cerr << "cannot open " << ssvfile << endl;
    exit(-1);
  }
  string line;
  vector<string> strs;
  size_t count = 0;
  size_t dimc;
  while (getline(in, line)) {
    if (strlen(line.c_str()) == 0) {
      continue;
    }
    strs.clear();
    split(line, ' ', strs);
    dimc = strs.size();
    count++;
    if (dim == 0) {
      dim = dimc;
    } else if (dim != dimc) {
      cerr << "# of dimension are different! " << dimc << endl;
      exit(1);
    }
  }
  in.close();
  return count;
}

void Timer::reset() { time = 0; }

void Timer::start() {
  reset();
  gettimeofday(&startTime, 0);
}

void Timer::restart() { gettimeofday(&startTime, 0); }

void Timer::stop() {
  gettimeofday(&stopTime, 0);
  sec = stopTime.tv_sec - startTime.tv_sec;
  usec = stopTime.tv_usec - startTime.tv_usec;
  if (usec < 0) {
    sec -= 1;
    usec += 1000000;
  }
  time += (double)sec + (double)usec / 1000000.0;
}

ostream &operator<<(ostream &os, Timer &t) {
  os << setprecision(6) << t.time << " (sec)";
  return os;
}
}
